#include <stdio.h>
#include <stdint.h>
#include <dc1394/dc1394.h>
#include <dc1394/camera.h>
#include <stdlib.h>
#include <inttypes.h>
#include "opencv2/video/tracking.hpp"
#include "opencv2/imgproc/imgproc.hpp"
#include "opencv2/highgui/highgui.hpp"
#include <iostream>
#include <ctype.h>

using namespace cv;
using namespace std;

#define IMAGE_FILE_NAME "image.pgm"

/*-----------------------------------------------------------------------
 *  Releases the cameras and exits
 *-----------------------------------------------------------------------*/
void cleanup_and_exit(dc1394camera_t *camera)
{
    dc1394_video_set_transmission(camera, DC1394_OFF);
    dc1394_capture_stop(camera);
    dc1394_camera_free(camera);
    exit(1);
}

int main(int argc, char *argv[])
{
    FILE* imagefile;
    dc1394camera_t *camera;
    unsigned int i;
    dc1394featureset_t features;
    dc1394framerate_t framerate = DC1394_FRAMERATE_15;
    dc1394video_mode_t video_mode = DC1394_VIDEO_MODE_FORMAT7_3;
    unsigned int width, height;
    dc1394video_frame_t *frame;
    dc1394camera_list_t * list;
    dc1394error_t err;
    dc1394_t * d;

    d = dc1394_new ();if (!d) return 1;err=dc1394_camera_enumerate (d, &list);DC1394_ERR_RTN(err,"Failed to enumerate cameras");
    printf("\nDetected cameras: %d\n",list->num);
    if (list->num == 0) {dc1394_log_error("No cameras found");return 1;}
    camera = dc1394_camera_new (d, list->ids[0].guid);if (!camera){dc1394_log_error("Failed to initialize camera %llx", list->ids[0].guid);return 1;}
    
    // Reset buses just in case...
    dc1394_reset_bus(camera);
    dc1394_camera_free(camera);
    dc1394_free (d);
    printf("Reseting bus...............................[OK]\n");

    d = dc1394_new ();
    err=dc1394_camera_enumerate (d, &list);
    DC1394_ERR_RTN(err,"Failed to enumerate cameras");

    if (list->num == 0) {dc1394_log_error("No cameras found");return 1;}
    
    camera = dc1394_camera_new (d, list->ids[0].guid);
    if (!camera) {dc1394_log_error("Failed to initialize camera with guid");return 1;}
    printf("Camera with GUID %"PRIx64"............[OK]\n", list->ids[0].guid);

    dc1394_camera_free_list (list);

    err=dc1394_video_set_iso_speed(camera, DC1394_ISO_SPEED_400);
    DC1394_ERR_CLN_RTN(err,cleanup_and_exit(camera),"Could not set iso speed");

    err=dc1394_video_set_mode(camera, video_mode);
    DC1394_ERR_CLN_RTN(err,cleanup_and_exit(camera),"Could not set video mode");

    err=dc1394_video_set_framerate(camera, framerate);
    DC1394_ERR_CLN_RTN(err,cleanup_and_exit(camera),"Could not set framerate");

    err=dc1394_capture_setup(camera,8, DC1394_CAPTURE_FLAGS_DEFAULT);
    DC1394_ERR_CLN_RTN(err,cleanup_and_exit(camera),"Could not setup camera-\nmake sure that the video mode and framerate are\nsupported by your camera");

    err=dc1394_format7_set_color_coding(camera,DC1394_VIDEO_MODE_FORMAT7_3,DC1394_COLOR_CODING_RAW16);
    DC1394_ERR_CLN_RTN(err,cleanup_and_exit(camera),"Could not set color coding");

    err=dc1394_feature_get_all(camera,&features);
    if (err!=DC1394_SUCCESS) {
        dc1394_log_warning("Could not get feature set");
    }
    else {
        //dc1394_feature_print_all(&features, stdout);
    }

    err=dc1394_video_set_transmission(camera, DC1394_ON);
    DC1394_ERR_CLN_RTN(err,cleanup_and_exit(camera),"Could not start camera iso transmission");

    dc1394_get_image_size_from_video_mode(camera, video_mode, &width, &height);
    unsigned char *leftImageBayer = new unsigned char[width * height];
    unsigned char *rightImageBayer = new unsigned char[width * height];
    unsigned char *leftImageRGB = new unsigned char[width * height * 3];
    unsigned char *rightImageRGB = new unsigned char[width * height * 3];
    IplImage* ColorD = cvCreateImage(cvSize(1024,768), IPL_DEPTH_8U ,3);        
    IplImage* ColorG = cvCreateImage(cvSize(1024,768), IPL_DEPTH_8U ,3);

    printf("\nCapturing frames from the cameras\n\nPRESS ANY KEY TO STOP\n\n");
    namedWindow( "Camera Droite", 0 );  
    namedWindow( "Camera Gauche", 0 );

    int c=-1;
  
    	while(c==-1)
    	{       

		    err=dc1394_capture_dequeue(camera, DC1394_CAPTURE_POLICY_WAIT, &frame);
		    DC1394_ERR_CLN_RTN(err,cleanup_and_exit(camera),"Could not capture a frame");

		    err=dc1394_capture_enqueue(camera, frame);
		    DC1394_ERR_CLN_RTN(err,cleanup_and_exit(camera),"Could not capture a frame");

		    // Deinterlace frames
		    for (i = 0; i < height * width; i++)
		    {
		    	leftImageBayer[i] = frame->image[i * 2];
		    	rightImageBayer[i] = frame->image[i * 2 + 1];
		    }
		    
		    // Convert frames
		    dc1394_bayer_decoding_8bit(leftImageBayer, leftImageRGB, width, height, DC1394_COLOR_FILTER_GBRG, DC1394_BAYER_METHOD_BILINEAR);
		    dc1394_bayer_decoding_8bit(rightImageBayer, rightImageRGB, width, height, DC1394_COLOR_FILTER_GBRG, DC1394_BAYER_METHOD_BILINEAR);

		    for (i = 0; i < (1024*768*3); i++)      
		    {
			ColorD->imageData[i] = (leftImageRGB[i]);      
		 	ColorG->imageData[i] = (rightImageRGB[i]);        
		    } 
		    cvShowImage("Camera 1",ColorD);
            	    cvShowImage("Camera 2",ColorG);
	
            	    c = (char)waitKey(10);//usleep(10);
	}

    cvReleaseImage(&ColorD);
    cvReleaseImage(&ColorG);

    err=dc1394_video_set_transmission(camera,DC1394_OFF);
    DC1394_ERR_CLN_RTN(err,cleanup_and_exit(camera),"Could not stop the camera");

    imagefile=fopen(IMAGE_FILE_NAME, "wb"); 
    if( imagefile == NULL) {perror( "Can't create '" IMAGE_FILE_NAME "'");cleanup_and_exit(camera);}
    fprintf(imagefile,"P6\n%u %u 255\n", width, height *2 );
    fwrite(leftImageRGB, 1, height*width*3, imagefile);
    fwrite(rightImageRGB, 1, height*width*3, imagefile);
    
    delete [] leftImageBayer;
    delete [] rightImageBayer;
    delete [] leftImageRGB;
    delete [] rightImageRGB;
    
    fclose(imagefile);
    printf("wrote: " IMAGE_FILE_NAME "\n");

    dc1394_video_set_transmission(camera, DC1394_OFF);
    dc1394_capture_stop(camera);
    dc1394_camera_free(camera);
    dc1394_free (d);

    return 0;
}


